package gregapi.oredict;

import static gregapi.data.CS.T;
import gregapi.code.HashSetNoNulls;

import java.util.HashMap;
import java.util.Set;

/**
 * @author Gregorius Techneticies
 * 
 * Used to process Non-Material based OreDict Stuff faster, like a String switch/case kind of a deal.
 */
public abstract class OreDictListenerEvent_Names implements IOreDictListenerEvent {
	private final HashMap<String, Set<IOreDictListenerEvent>> NAME_EVENTS = new HashMap<String, Set<IOreDictListenerEvent>>();
	
	public OreDictListenerEvent_Names(OreDictPrefix aPrefix) {
		addAllListeners();
		aPrefix.addListener(this);
	}
	
	public OreDictListenerEvent_Names() {
		addAllListeners();
		OreDictManager.INSTANCE.addListener(this);
		for (String tOreDictName : NAME_EVENTS.keySet()) OreDictManager.INSTANCE.addNotUnknownName(tOreDictName);
	}
	
	public abstract void addAllListeners();
	
	public boolean addListener(Object aOreDictName, IOreDictListenerEvent aListener) {
		aOreDictName = aOreDictName.toString();
		Set<IOreDictListenerEvent> tSet = NAME_EVENTS.get(aOreDictName);
		if (tSet == null) NAME_EVENTS.put((String)aOreDictName, tSet = new HashSetNoNulls<IOreDictListenerEvent>());
		if (!tSet.add(aListener)) throw new UnknownError("Unknown Error: " + aOreDictName + "  -  " + aListener.toString());
		return T;
	}
	public boolean addListener(Object aOreDictName1, Object aOreDictName2, IOreDictListenerEvent aListener) {
		return addListener(aOreDictName1, aListener) && addListener(aOreDictName2, aListener);
	}
	public boolean addListener(Object aOreDictName1, Object aOreDictName2, Object aOreDictName3, IOreDictListenerEvent aListener) {
		return addListener(aOreDictName1, aListener) && addListener(aOreDictName2, aListener) && addListener(aOreDictName3, aListener);
	}
	public boolean addListener(Object aOreDictName1, Object aOreDictName2, Object aOreDictName3, Object aOreDictName4, IOreDictListenerEvent aListener) {
		return addListener(aOreDictName1, aListener) && addListener(aOreDictName2, aListener) && addListener(aOreDictName3, aListener) && addListener(aOreDictName4, aListener);
	}
	public boolean addListener(Object[] aOreDictNames, IOreDictListenerEvent aListener) {
		for (Object tOreDictName : aOreDictNames) addListener(tOreDictName, aListener);
		return T;
	}
	
	@Override
	public void onOreRegistration(OreDictRegistrationContainer aEvent) {
		Set<IOreDictListenerEvent> tSet = NAME_EVENTS.get(aEvent.mOreDictName);
		if (tSet != null) for (IOreDictListenerEvent tListener : tSet) tListener.onOreRegistration(aEvent);
	}
}